pub struct Angle {
pub group: Group,
pub line_a: EntityHandle<LineSegment>,
pub line_b: EntityHandle<LineSegment>,
pub angle: f64,
pub workplane: Option<EntityHandle<Workplane>>,
pub supplementary: bool,
}Expand description
Constrain the angle between line_a and line_b, in degrees.
This constraint equation is written in the form
(A dot B)/(|A||B|) = cos(valA)where A and B are vectors in the directions of lines A and B. This equation
does not specify the angle unambiguously; for example, note that valA = +/- 90
degrees will produce the same equation.
If supplementary is true, then the constraint is instead that
(A dot B)/(|A||B|) = -cos(valA)Note that the solver will fail if the two lines are initially parallel to each other.
Fields
group: GroupThe group that Angle belongs to.
line_a: EntityHandle<LineSegment>line_b: EntityHandle<LineSegment>angle: f64The angle between line_a and line_b, in degrees.
workplane: Option<EntityHandle<Workplane>>If provided, constraint applies when projected onto this workplane.
supplementary: boolIf true, sets the supplementary angle.
Implementations
Sourceimpl Angle
impl Angle
Sourcepub fn new(
group: Group,
line_a: EntityHandle<LineSegment>,
line_b: EntityHandle<LineSegment>,
angle: f64,
workplane: Option<EntityHandle<Workplane>>,
supplementary: bool,
) -> Self
pub fn new( group: Group, line_a: EntityHandle<LineSegment>, line_b: EntityHandle<LineSegment>, angle: f64, workplane: Option<EntityHandle<Workplane>>, supplementary: bool, ) -> Self
Create a new Angle instance.