Struct PtLineDistance

pub struct PtLineDistance {
    pub group: Group,
    pub point: EntityHandle<Point>,
    pub line: EntityHandle<LineSegment>,
    pub distance: f64,
    pub workplane: Option<EntityHandle<Workplane>>,
}
The distance between point and line is equal to distance.

If the constraint is projected onto a workplane, distance is a signed distance. Positive vs. negative distance correspond to a point that is above vs. below the line.

If no workplane is provided, then distance must always be positive.

Fields

group: Group

The group that PtLineDistance belongs to.

point: EntityHandle<Point>line: EntityHandle<LineSegment>distance: f64workplane: Option<EntityHandle<Workplane>>

If provided, constraint applies when projected onto this workplane.

Implementations

Source

impl PtLineDistance

pub fn new( group: Group, point: EntityHandle<Point>, line: EntityHandle<LineSegment>, distance: f64, workplane: Option<EntityHandle<Workplane>>, ) -> Self

Create a new PtLineDistance instance.